@@ -23,7 +23,7 @@
 #include "dialog_progress_creating.h"
 #include "translation/dialog_change_language.h"
 #include "translation/window_translations.h"
-#include "utils.h"
+#include <glom/libglom/utils.h>
 #include <libgnome/gnome-help.h> //For gnome_help_display
 #include "config.h" //For VERSION.
 #include <cstdio>
@@ -42,6 +42,9 @@ App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml
   m_menu_tables_ui_merge_id(0),
   m_menu_reports_ui_merge_id(0)
 {
+  //Show the icon in the window manager's window title bar and in the list of running applications:
+  set_icon_from_file(GLOM_ICON_DIR "/glom.png");
+
   //Load widgets from glade file:
   refGlade->get_widget("bakery_vbox", m_pBoxTop);
   refGlade->get_widget_derived("vbox_frame", m_pFrame); //This one is derived. There's a lot happening here.
@@ -84,7 +87,7 @@ bool App_Glom::init(const Glib::ustring& document_uri)
   }
   else
   {
-    bool test = open_document(document_uri);
+    const bool test = open_document(document_uri);
     if(!test)
       return offer_new_or_existing();
   }
@@ -378,7 +381,9 @@ void App_Glom::init_create_document()
 {
   if(!m_pDocument)
   {
-    m_pDocument = new Document_Glom();
+    Document_Glom* document_glom = new Document_Glom();
+    m_pDocument = document_glom;
+    document_glom->set_parent_window(this); //So that it can show a BusyCursor when loading and saving.
 
     //Tell document about view:
     m_pDocument->set_view(m_pFrame);
@@ -433,6 +438,7 @@ bool App_Glom::on_document_load()
             return false;
           else
           {
+            pDocument->set_file_uri(Glib::ustring()); //Prevent it from defaulting to the read-only examples directory when offering saveas.
             offer_saveas();
             if(get_operation_cancelled())
               return false;
@@ -470,7 +476,7 @@ bool App_Glom::on_document_load()
         connection_pool->set_user(pDocument->get_connection_user());
         connection_pool->set_database(pDocument->get_connection_database());
 
-        connection_pool->set_ready_to_connect(); //Box_DB::connect_to_server() will now attempt the connection-> Shared instances of m_Connection will also be usable.
+        connection_pool->set_ready_to_connect(this); //Box_DB::connect_to_server() will now attempt the connection-> Shared instances of m_Connection will also be usable.
 
         //Attempt to connect to the specified database:
         try
@@ -495,23 +501,21 @@ bool App_Glom::on_document_load()
               //If the database was not successfully recreated:
               if(!user_cancelled)
               {
-                //Tell the user:
-                Gtk::Dialog* dialog = 0;
-                try
-                {
-                  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_error_create_database");
-                  refXml->get_widget("dialog_error_create_database", dialog);
-                  dialog->set_transient_for(*this);
-                  dialog->run();
-                  delete dialog;
-                }
-                catch(const Gnome::Glade::XmlError& ex)
-                {
-                  std::cerr << ex.what() << std::endl;
-                }
+                //Let the user try again.
+                //A warning has already been shown.
+                return offer_new_or_existing();
               }
-
-              return false;
+              else
+                return false;
+            }
+            else
+            {
+              //Make sure that the changes (mark as non example, and save the new database name) are really saved:
+              //Change the user level temporarily so that save_changes() actually saves:
+              const AppState::userlevels user_level = pDocument->get_userlevel();
+              pDocument->set_userlevel(AppState::USERLEVEL_DEVELOPER);
+              pDocument->set_modified(true);
+              pDocument->set_userlevel(user_level); //Change it back.
             }
           }
         }
@@ -723,7 +727,8 @@ bool App_Glom::offer_new_or_existing()
                 }
                 else
                 {
-                  return false;
+                  //Ask again:
+                  return offer_new_or_existing();
                 }
               }
               else
@@ -769,7 +774,7 @@ void App_Glom::set_mode_data()
 
 void App_Glom::set_mode_find()
 {
-  m_action_mode_data->activate();
+  m_action_mode_find->activate();
 }
 
 
@@ -874,7 +879,7 @@ bool App_Glom::recreate_database(bool& user_cancelled)
     connection_pool->set_database(db_name); //Specify the new database when connecting from now on.
 
   dialog_progress->pulse();
-  Bakery::BusyCursor(*this);
+  Bakery::BusyCursor busy_cursor(this);
 
   sharedptr<SharedConnection> sharedconnection;
   try
@@ -916,10 +921,9 @@ bool App_Glom::recreate_database(bool& user_cancelled)
       //Create the developer group, and make this user a member of it:
       //If we got this far then the user must really have developer privileges already:
       dialog_progress->pulse();
-      m_pFrame->add_standard_groups();
-
-      Glib::ustring strQuery = "ALTER GROUP " GLOM_STANDARD_GROUP_NAME_DEVELOPER " ADD USER " + connection_pool->get_user();
-      m_pFrame->Query_execute(strQuery);
+      const bool test = m_pFrame->add_standard_groups();
+      if(!test)
+        return false;
 
       //Add any example data to the table:
       dialog_progress->pulse();
@@ -1154,7 +1158,9 @@ void App_Glom::on_menu_file_save_as_example()
         //std::cout << "debug: table_name=" << table_name << std::endl;
 
         Glib::ustring row_text;
-        m_pFrame->export_data_to_string(row_text, table_name, sequence);
+        FoundSet found_set;
+        found_set.m_table_name = table_name;
+        m_pFrame->export_data_to_string(row_text, found_set, sequence);
         //std::cout << "  debug after row_text=" << row_text << std::endl;
 
         document->set_table_example_data(table_name, row_text);
