@@ -20,8 +20,8 @@
  
 #include "dialog_database_preferences.h"
 #include "box_db.h" //For Box_DB::connect_to_server().
-#include "standard_table_prefs_fields.h"
-#include "data_structure/glomconversions.h"
+#include <glom/libglom/standard_table_prefs_fields.h>
+#include <glom/libglom/data_structure/glomconversions.h>
 #include <bakery/Utilities/BusyCursor.h>
 #include <glibmm/i18n.h>
 
@@ -98,7 +98,7 @@ void Dialog_Database_Preferences::on_treeview_cell_edited_next_value(const Glib:
         " WHERE \"" GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_TABLE_NAME "\" = '" + table_name + "' AND "
                "\"" GLOM_STANDARD_TABLE_AUTOINCREMENTS_FIELD_FIELD_NAME "\" = '" + field_name +"'";
 
-    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query, this);
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(sql_query, this);
     if(!datamodel)
     {
       g_warning("Dialog_Database_Preferences::on_treeview_cell_edited_next_value(): UPDATE failed.");
@@ -141,7 +141,7 @@ void Dialog_Database_Preferences::load_from_document()
 
   NumericFormat numeric_format; //ignored
 
-  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query, this);
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(sql_query, this);
   guint count = datamodel->get_n_rows();
   for(guint i = 0; i < count; ++i)
   {
