@@ -83,7 +83,7 @@ protected:
   ///Fill the existing layout with data from the database.
   virtual bool fill_from_database(); //override.
 
-  virtual void refresh_related_fields(const FieldInRecord& field_in_record_changed, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
+  virtual void refresh_related_fields(const LayoutFieldInRecord& field_in_record_changed, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
 
   virtual type_vecLayoutFields get_fields_to_show() const;
   //virtual Glib::ustring build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause);
@@ -100,10 +100,17 @@ protected:
 
   /** Get the fields that are in related tables, via a relationship using @a field_name changes.
   */
-  type_vecLayoutFields get_related_fields(const Glib::ustring& field_name) const;
+  type_vecLayoutFields get_related_fields(const sharedptr<const LayoutItem_Field>& field) const;
+
+  bool record_delete(const Gnome::Gda::Value& primary_key_value);
+
+  ///This allows record_new() to set the generated/entered primary key value, needed by Box_Data_List:
+  virtual void set_primary_key_value(const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& value);
+
+  ///New record with all entered field values.
+  Glib::RefPtr<Gnome::Gda::DataModel> record_new(bool use_entered_data = true, const Gnome::Gda::Value& primary_key_value = Gnome::Gda::Value()); 
+  Glib::RefPtr<Gnome::Gda::DataModel> record_new(bool use_entered_data, const Gnome::Gda::Value& primary_key_value, const Gtk::TreeModel::iterator& row);
 
-  virtual bool record_delete(const Gnome::Gda::Value& primary_key_value);
-  virtual Glib::RefPtr<Gnome::Gda::DataModel> record_new(bool use_entered_data = true, const Gnome::Gda::Value& primary_key_value = Gnome::Gda::Value()); //New record with all entered field values.
   Gnome::Gda::Value generate_next_auto_increment(const Glib::ustring& table_name, const Glib::ustring field_name);
 
   virtual sharedptr<Field> get_field_primary_key() const = 0;
