@@ -23,7 +23,7 @@
 #include "gdk/gdktypes.h"
 
 FlowTable::FlowTableItem::FlowTableItem()
-: m_expand_first(false),
+: m_expand_first_full(false),
   m_expand_second(false)
 {
 }
@@ -50,15 +50,10 @@ void FlowTable::set_design_mode(bool value)
 
 void FlowTable::add(Gtk::Widget& first, Gtk::Widget& second, bool expand_second)
 {
-if(expand_second)
-{
-  g_warning("FlowTable::add: expand_second");
-}
-
   FlowTableItem item;
   item.m_first = &first;
   item.m_second = &second;
-  item.m_expand_second = expand_second;
+  item.m_expand_second = expand_second; //Expand to fill the width for all of the second item.
   m_children.push_back(item);
 
   gtk_widget_set_parent(first.gobj(), GTK_WIDGET(gobj()));
@@ -69,7 +64,7 @@ void FlowTable::add(Gtk::Widget& first, bool expand)
 {
   FlowTableItem item;
   item.m_first = &first;
-  item.m_expand_first = expand;
+  item.m_expand_first_full = expand; //Expand to fill the width for first and second.
   item.m_second = 0;
   m_children.push_back(item);
 
@@ -230,7 +225,7 @@ int FlowTable::get_minimum_column_height(guint start_widget, guint columns_count
 
     const guint count_items_remaining = m_children.size() - start_widget;
 
-    for(guint first_column_widgets_count = 1;  first_column_widgets_count < count_items_remaining; ++first_column_widgets_count)
+    for(guint first_column_widgets_count = 1;  first_column_widgets_count <= count_items_remaining; ++first_column_widgets_count)
     {
       int first_column_width = 0;
       int first_column_height = get_column_height(start_widget, first_column_widgets_count, first_column_width);
@@ -495,8 +490,9 @@ void FlowTable::on_size_allocate(Gtk::Allocation& allocation)
       {
         //Assign space to the child:
 
-        //Make all the left edges line up:
-        item.m_first_allocation = assign_child(first, column_x_start, column_child_y_start);
+        //Make all the left edges line up, and give the full first-item width, even if that's more width than it requested,
+        //so that it can align itself in the available space.
+        item.m_first_allocation = assign_child(first, column_x_start, column_child_y_start, first_max_width, item_height);
         something_added = true;
       }
 
@@ -510,7 +506,7 @@ void FlowTable::on_size_allocate(Gtk::Allocation& allocation)
           item.m_second_allocation = assign_child(second, column_x_start_second, column_child_y_start);
         else
         {
-          int second_width_available = column_max_width - first_max_width - m_padding;
+          const int second_width_available = column_max_width - first_max_width - m_padding;
           item.m_second_allocation = assign_child(second, column_x_start_second, column_child_y_start, second_width_available, item_height);
         }
 
@@ -522,7 +518,7 @@ void FlowTable::on_size_allocate(Gtk::Allocation& allocation)
       //There is only one item - so let it take up the whole width of the column, if requested:
 
       //Assign space to the child:
-      if(!(item.m_expand_first))
+      if(!(item.m_expand_first_full))
         item.m_first_allocation = assign_child(first, column_x_start, column_child_y_start);
       else
       {
@@ -647,7 +643,7 @@ void FlowTable::remove(Gtk::Widget& first)
   {
     if((iter->m_first == &first) && (iter->m_second == 0))
     {
-      g_warning("FlowTable::remove(): removing %10X", (guint)&first);
+      //g_warning("FlowTable::remove(): removing %10X", (guint)&first);
 
       m_children.erase(iter);
       break;
