@@ -31,6 +31,8 @@
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include <glibmm/i18n.h>
 
+
+
 FlowTableWithFields::Info::Info()
 : m_first(0),
   m_second(0),
@@ -271,7 +273,8 @@ void FlowTableWithFields::add_layout_notebook_at_position(const sharedptr<Layout
       if(portal)
       {
         const Glib::ustring tab_title = glom_get_sharedptr_title_or_name(portal->get_relationship());
-        tab_label->set_markup(Bakery::App_Gtk::util_bold_message(tab_title));
+        //tab_label->set_markup(Bakery::App_Gtk::util_bold_message(tab_title));
+        tab_label->set_label(tab_title);
 
         //Add a Related Records list for this portal:
         Box_Data_List_Related* portal_box = create_related(portal, false /* no label, because it's in the tab instead. */);
@@ -284,7 +287,8 @@ void FlowTableWithFields::add_layout_notebook_at_position(const sharedptr<Layout
       else
       {
         const Glib::ustring tab_title = group->get_title_or_name();
-        tab_label->set_markup(Bakery::App_Gtk::util_bold_message(tab_title));
+        //tab_label->set_markup(Bakery::App_Gtk::util_bold_message(tab_title));
+        tab_label->set_label(tab_title);
 
         //Add a FlowTable for this group:
         FlowTableWithFields* flow_table = Gtk::manage( new FlowTableWithFields() );
@@ -408,19 +412,16 @@ void FlowTableWithFields::add_field_at_position(const sharedptr<LayoutItem_Field
   info.m_second = pDataWidget; 
   info.m_second->show_all();
 
-  info.m_first = Gtk::manage(new Gtk::Alignment());
-
   //Add a label, if one is necessary:
   Gtk::Label* label = info.m_second->get_label();
+  info.m_first = label;
   if(label && !label->get_text().empty())
   {
-    info.m_first->add( *label );
-    label->set_alignment(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER); //Align right.
+    label->property_xalign() = 0.0f; //Equivalent to Gtk::ALIGN_LEFT, but we can't use that here.
+    label->property_yalign() = 0.5f; //Equivalent ot Gtk::ALIGN_CENTER, but we can't use that here.; 
     label->show();
 
-    info.m_first->show();
-    info.m_first->set(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER);
-    info.m_first->show_all_children(); //This does not seem to work, so we show the label explicitly.
+    label->show();
   }
 
   //info.m_group = layoutitem_field.m_group;
@@ -428,7 +429,16 @@ void FlowTableWithFields::add_field_at_position(const sharedptr<LayoutItem_Field
   //Expand multiline text fields to take up the maximum possible width:
   bool expand_second = false;
   if( (layoutitem_field->get_glom_type() == Field::TYPE_TEXT) && layoutitem_field->get_formatting_used().get_text_format_multiline())
+  {
     expand_second = true;
+    if(label)
+      label->property_yalign() = 0.0f; //Equivalent to Gtk::ALIGN_TOP. Center is neater next to entries, but center is silly next to multi-line text boxes.
+  }
+  else if(layoutitem_field->get_glom_type() == Field::TYPE_IMAGE)
+  {
+    if(label)
+      label->property_yalign() = 0.0f; //Equivalent to Gtk::ALIGN_TOP. Center is neater next to entries, but center is silly next to large images.
+  }
 
   add(*(info.m_first), *(info.m_second), expand_second);
 
