@@ -21,7 +21,7 @@
 #ifndef GLOM_MODE_DATA_LAYOUT_WIDGET_BASE_H
 #define GLOM_MODE_DATA_LAYOUT_WIDGET_BASE_H
 
-#include "../data_structure/layout/layoutitem.h"
+#include <glom/libglom/data_structure/layout/layoutitem.h>
 #include <gtkmm.h>
 #include "../mode_data/treestore_layout.h" //Forthe enum.
 
@@ -40,17 +40,27 @@ public:
   sharedptr<const LayoutItem> get_layout_item() const;
   sharedptr<LayoutItem> get_layout_item();
 
+  enum enumType
+  {
+    TYPE_FIELD,
+    TYPE_GROUP,
+    TYPE_NOTEBOOK,
+    TYPE_PORTAL,
+    TYPE_BUTTON,
+    TYPE_TEXT
+  };
+
   //Popup-menu:
   virtual void setup_menu();
   virtual void on_menupopup_activate_layout();
   virtual void on_menupopup_activate_layout_properties();
-  virtual void on_menupopup_add_item(TreeStore_Layout::enumType item);
+  virtual void on_menupopup_add_item(enumType item);
 
 
   typedef sigc::signal<void> type_signal_layout_changed;
   type_signal_layout_changed signal_layout_changed();
 
-  typedef sigc::signal<void, TreeStore_Layout::enumType> type_signal_layout_item_added;
+  typedef sigc::signal<void, enumType> type_signal_layout_item_added;
   type_signal_layout_item_added signal_layout_item_added();
 
   //Allow a child widget to delegate to a parent widget:
@@ -61,6 +71,8 @@ public:
   typedef sigc::signal<void> type_signal_user_requested_layout_properties;
   type_signal_user_requested_layout_properties signal_user_requested_layout_properties();
 
+  virtual void set_read_only(bool read_only = true);
+
 protected:
   virtual App_Glom* get_application() const; // = 0;
 
@@ -74,9 +86,11 @@ protected:
   type_signal_user_requested_layout_properties m_signal_user_requested_layout_properties;
 
   Gtk::Menu* m_pMenuPopup;
+
+  //TODO_Performance: //Presumably we waste lots of memory by having this in each layout widget. Maybe we can use one shared menu.
   Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
   Glib::RefPtr<Gtk::UIManager> m_refUIManager;
-  Glib::RefPtr<Gtk::Action> m_refContextLayout, m_refContextLayoutProperties, m_refContextAddField, m_refContextAddRelatedRecords, m_refContextAddGroup;
+  Glib::RefPtr<Gtk::Action> m_refContextLayout, m_refContextLayoutProperties, m_refContextAddField, m_refContextAddRelatedRecords, m_refContextAddGroup, m_refContextAddNotebook, m_refContextAddButton, m_refContextAddText;
 
 };
 
